@@ -60,6 +60,8 @@ class MuminDataset:
             Whether to include hashtags in the dataset. Defaults to True.
         include_mentions (bool, optional):
             Whether to include mentions in the dataset. Defaults to True.
+        include_timelines (bool, optional):
+            Whether to include timelines in the dataset. Defaults to False.
         text_embedding_model_id (str, optional):
             The HuggingFace Hub model ID to use when embedding texts. Defaults
             to 'xlm-roberta-base'.
@@ -80,6 +82,7 @@ class MuminDataset:
         include_extra_images (bool): Whether to include user/article images.
         include_hashtags (bool): Whether to include hashtags.
         include_mentions (bool): Whether to include mentions.
+        include_timelines (bool): Whether to include timelines.
         size (str): The size of the dataset.
         dataset_path (pathlib Path): The dataset file.
         text_embedding_model_id (str): The model ID used for embedding text.
@@ -96,7 +99,7 @@ class MuminDataset:
               Network Posts_ (2021)
     '''
     download_url: str = ('https://github.com/CLARITI-REPHRAIN/mumin-build/raw/'
-                         '7f54a4f805a0489b772cab5480bc9cce95d52660'
+                         '7cdf5c5ea3689af597eeb1f0d39c3412ea30c3b7'
                          '/data/mumin.zip')
     _node_dump: List[str] = ['claim', 'tweet', 'user', 'image', 'article',
                              'hashtag', 'reply']
@@ -127,6 +130,7 @@ class MuminDataset:
                  include_extra_images: bool = False,
                  include_hashtags: bool = True,
                  include_mentions: bool = True,
+                 include_timelines: bool = False,
                  text_embedding_model_id: str = 'xlm-roberta-base',
                  image_embedding_model_id: str = ('google/vit-base-patch16-'
                                                   '224-in21k'),
@@ -139,6 +143,7 @@ class MuminDataset:
         self.include_extra_images = include_extra_images
         self.include_hashtags = include_hashtags
         self.include_mentions = include_mentions
+        self.include_timelines = include_timelines
         self.text_embedding_model_id = text_embedding_model_id
         self.image_embedding_model_id = image_embedding_model_id
         self.verbose = verbose
@@ -397,8 +402,15 @@ class MuminDataset:
         claim_df = self.nodes['claim']
         discusses_rel = self.rels[('tweet', 'discusses', 'claim')]
         include_claim = claim_df.id.isin(discusses_rel.tgt.tolist())
-        self.nodes['claim'] = (claim_df[include_claim]
-                               .reset_index(drop=True))
+        self.nodes['claim'] = claim_df[include_claim].reset_index(drop=True)
+
+        # Filter timeline tweets
+        if not self.include_timelines:
+            src_tweet_ids = (self.rels[('tweet', 'discusses', 'claim')]
+                                 .src
+                                 .tolist())
+            is_src = self.nodes['tweet'].tweet_id.isin(src_tweet_ids)
+            self.nodes['tweet'] = self.nodes['tweet'].loc[is_src]
 
         return self
 
@@ -431,15 +443,23 @@ class MuminDataset:
             # Store any features the nodes might have had before hydration
             prehydration_df = self.nodes[node_type].copy()
 
-            # Rehydrate the tweets
-            if node_type == 'tweet':
-                src_dict = dict(tweet_ids=source_tweet_ids)
-                tweet_dfs = self._twitter.rehydrate_tweets(tweet_ids=tweet_ids)
-                source_tweet_dfs = self._twitter.rehydrate_tweets(**src_dict)
-            else:
-                tweet_dfs = self._twitter.rehydrate_tweets(tweet_ids=tweet_ids)
-                source_tweet_dfs = {key: pd.DataFrame()
-                                    for key in tweet_dfs.keys()}
+            # Rehydrate the source tweets
+            if len(source_tweet_ids) > 0:
+                params = dict(tweet_ids=source_tweet_ids)
+                source_tweet_dfs = self._twitter.rehydrate_tweets(**params)
+
+                if len(tweet_ids) == 0:
+                    tweet_dfs = {key: pd.DataFrame()
+                                 for key in source_tweet_dfs.keys()}
+
+            # Rehydrate the other tweets
+            if len(tweet_ids) > 0:
+                params = dict(tweet_ids=tweet_ids)
+                tweet_dfs = self._twitter.rehydrate_tweets(**params)
+
+                if len(source_tweet_ids) == 0:
+                    source_tweet_dfs = {key: pd.DataFrame()
+                                        for key in tweet_dfs.keys()}
 
             # Extract and store tweets and users
             tweet_df = pd.concat([source_tweet_dfs['tweets'],
@@ -627,7 +647,7 @@ class MuminDataset:
         for rel_type, rel_df in self.rels.items():
 
             # Pop the relation if the dataframe does not exist
-            if rel_df is None:
+            if rel_df is None or len(rel_df) == 0:
                 rels_to_pop.append(rel_type)
                 continue
 
@@ -708,15 +728,16 @@ class MuminDataset:
                                        for col, dtype in dtype_dict.items()
                                        if dtype != 'numpy'}
                 for col, dtype in dtype_dict_no_numpy.items():
-                    self.nodes[ntype][col] = (self.nodes[ntype][col]
-                                              .astype(dtype))
+                    if col in self.nodes[ntype].columns:
+                        self.nodes[ntype][col] = (self.nodes[ntype][col]
+                                                  .astype(dtype))
 
                 # For numpy columns, set the type manually
                 def numpy_fn(x):
                     return np.asarray(x)
 
                 for col, dtype in dtype_dict.items():
-                    if dtype == 'numpy':
+                    if dtype == 'numpy' and col in self.nodes[ntype].columns:
                         self.nodes[ntype][col] = (self.nodes[ntype][col]
                                                       .map(numpy_fn))
 
